@@ -278,6 +278,40 @@ START_TEST(test_drop_ideo)
 }
 END_TEST
 
+START_TEST(test_url)
+{
+	init(NULL, IGNORE_SPACE);
+	start(T("http PTRCKPRRY http://www.PTRCKPRRY.com/"));
+	assert_text_eq(next_type(), T("http"));
+	assert_text_eq(next_type(), TYPE_IGNORE);
+	assert_text_eq(next_type(), T("ptrckprry"));
+	assert_text_eq(next_type(), TYPE_IGNORE);
+	assert_text_eq(next_type(), T("http://www.ptrckprry.com/"));
+	assert_text_eq(next_type(), TYPE_EOT);
+}
+END_TEST
+
+
+START_TEST(test_hashtag)
+{
+	init(NULL, IGNORE_SPACE);
+	start(T("#useR2017"));
+	assert_text_eq(next_type(), T("#user2017"));
+	assert_text_eq(next_type(), TYPE_EOT);
+}
+END_TEST
+
+
+START_TEST(test_mention)
+{
+	init(NULL, IGNORE_SPACE);
+	start(T("@PtrckPrry"));
+	assert_text_eq(next_type(), T("@ptrckprry"));
+	assert_text_eq(next_type(), TYPE_EOT);
+}
+END_TEST
+
+
 
 Suite *filter_suite(void)
 {
@@ -292,6 +326,9 @@ Suite *filter_suite(void)
         tcase_add_test(tc, test_combine);
         tcase_add_test(tc, test_basic_census);
         tcase_add_test(tc, test_drop_ideo);
+        tcase_add_test(tc, test_url);
+        tcase_add_test(tc, test_hashtag);
+        tcase_add_test(tc, test_mention);
 	suite_add_tcase(s, tc);
 
 	return s;
