@@ -24,6 +24,7 @@
 
 static const R_CallMethodDef CallEntries[] = {
 	CALLDEF(abbreviations, 1),
+	CALLDEF(alloc_text_handle, 0),
 	CALLDEF(anyNA_text, 1),
 	CALLDEF(as_character_json, 1),
 	CALLDEF(as_character_text, 1),
@@ -31,8 +32,8 @@ static const R_CallMethodDef CallEntries[] = {
 	CALLDEF(as_double_json, 1),
 	CALLDEF(as_list_json, 2),
 	CALLDEF(as_logical_json, 1),
-	CALLDEF(as_text_character, 1),
-	CALLDEF(as_text_json, 1),
+	CALLDEF(as_text_character, 2),
+	CALLDEF(as_text_json, 2),
 	CALLDEF(dim_json, 1),
 	CALLDEF(format_text, 9),
 	CALLDEF(is_na_text, 1),
@@ -50,26 +51,29 @@ static const R_CallMethodDef CallEntries[] = {
 	CALLDEF(stopwords, 1),
 	CALLDEF(subscript_json, 2),
 	CALLDEF(subset_json, 3),
-	CALLDEF(subset_text_handle, 2),
 	CALLDEF(term_counts_text, 8),
 	CALLDEF(term_matrix_text, 5),
+	CALLDEF(text_c, 3),
 	CALLDEF(text_count, 2),
 	CALLDEF(text_detect, 2),
-	CALLDEF(text_filter_update, 1),
+	CALLDEF(text_length, 1),
 	CALLDEF(text_locate, 2),
+	CALLDEF(text_match, 2),
 	CALLDEF(text_nsentence, 1),
 	CALLDEF(text_ntoken, 1),
 	CALLDEF(text_ntype, 2),
 	CALLDEF(text_split_sentences, 2),
 	CALLDEF(text_split_tokens, 2),
+	CALLDEF(text_sub, 3),
 	CALLDEF(text_tokens, 1),
 	CALLDEF(text_types, 2),
 	CALLDEF(text_valid, 1),
 	CALLDEF(utf8_coerce, 1),
 	CALLDEF(utf8_encode, 3),
 	CALLDEF(utf8_format, 9),
+	CALLDEF(utf8_normalize, 5),
 	CALLDEF(utf8_valid, 1),
-	CALLDEF(utf8_width, 2),
+	CALLDEF(utf8_width, 3),
         {NULL, NULL, 0}
 };
 
