@@ -1,4 +1,4 @@
-context("text_length")
+context("text_nunit")
 
 
 test_that("text_nsentence can works on sentences", {
@@ -14,6 +14,11 @@ test_that("text_nsentence can works on sentences", {
 })
 
 
+test_that("text_nsentence handles NA and empty", {
+    expect_equal(text_nsentence(c(NA, "")), c(NA, 0))
+})
+
+
 test_that("text_ntoken can works on tokens", {
     text <- c(a="He said, 'Are you going?' John Shook his head.",
               b="'Are you going?' John asked",
@@ -27,9 +32,17 @@ test_that("text_ntoken can works on tokens", {
 })
 
 
-test_that("text_ntype works on types", {
-    expect_equal(text_ntype(LETTERS, collapse = TRUE), 26)
+test_that("text_ntoken handles NA and empty", {
+    expect_equal(text_ntoken(c(NA, "")), c(NA, 0))
+})
 
-    expect_equal(text_ntype(paste(LETTERS, letters, LETTERS)),
-                 rep(1, 26))
+
+test_that("text_length includes NAs, text_ntoken doesn't", {
+    x <- c("A man, a plan.", "A \"canal\"?", "Panama!")
+    f <- text_filter(drop_punct = TRUE)
+    toks <- text_tokens(x, f)
+    ntok <- text_ntoken(x, f)
+    len <- text_length(x, f)
+    expect_equal(len, sapply(toks, length))
+    expect_equal(ntok, len - sapply(toks, function(t) sum(is.na(t))))
 })
